@@ -1,4 +1,5 @@
-# Copyright (C) 1998-2014 ABINIT group (XG)
+#!/usr/bin/env bash
+# Copyright (C) 1998-2016 ABINIT group (XG)
 # 
 # The purpose of this script is to change some
 # expression by another in a whole set of files
@@ -8,7 +9,7 @@ for file in "$@"
 do
  echo "working on $file"
  rm -f tmp.file
- sed -e 's!_custom_!_custom!' $file > tmp.file
+ sed -e 's!t113!t88!' $file > tmp.file
  echo "changes done "
  # put the modified file at the correct place
  mv tmp.file $file
