@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+from __future__ import print_function
 
 import os
 import re
@@ -13,7 +14,8 @@ def main(top):
   for root,dirs,files in os.walk(top):
     for src in files:
       if ( fortran.search(src) ):
-        code = file(os.path.join(root,src),"r").readlines()
+        with open(os.path.join(root,src), "r") as fh:
+            code = fh.readlines()
 
         for line in code:
           if ( cppline.match(line) ):
@@ -28,13 +30,14 @@ def main(top):
               else:
                 cppopts[item] = 1
 
-  print "Option                             Occurences"
-  print "--------------------------------   ----------"
-  names = cppopts.keys()
-  names.sort()
+  print ("Option                             Occurences")
+  print ("--------------------------------   ----------")
+  names = sorted(cppopts.keys())
   for opt in names:
-    print "%-32s   %10d" % (opt,cppopts[opt])
-  print ""
+    print ("%-32s   %10d" % (opt,cppopts[opt]))
+  print ("")
+
+  return 0
 
 if __name__ == "__main__":
 
