@@ -1,4 +1,4 @@
-# Copyright (C) 2010-2014 ABINIT group (Yann Pouillon)
+# Copyright (C) 2010-2016 ABINIT group (Yann Pouillon)
 # This file is distributed under the terms of the
 # GNU General Public License, see ~abinit/COPYING
 # or http://www.gnu.org/copyleft/gpl.txt .
@@ -55,7 +55,7 @@ cat > ${module_name}.F90 <<EOF
 module ${module_name}
     
  use defs_basis
- use m_profiling
+ use m_profiling_abi
  use m_errors
 
  implicit none
