@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2014 ABINIT group (XG)
+# Copyright (C) 1998-2016 ABINIT group (XG)
 # This file is distributed under the terms of the
 # GNU General Public License, see ~abinit/COPYING
 # or http://www.gnu.org/copyleft/gpl.txt .
@@ -63,7 +63,7 @@ subroutine ${routine_name}(argin,argout,option,sizein,sizeout)
     
  use defs_basis
  use m_errors
- use m_profiling
+ use m_profiling_abi
 
  implicit none
 
