@@ -3,6 +3,7 @@ document.write(' Files that describe the ABINIT input variables:<br> ');
 document.write(' <ul> ');
 document.write(' <li> Basic variables, <A href="varbas.html">VARBAS</a></li> ');
 document.write(' <li> Developper variables, <A href="vardev.html">VARDEV</a></li> ');
+document.write(' <li> Electron-Phonon variables, <A href="vareph.html">VAREPH</a></li> ');
 document.write(' <li> Finite field and polarization variables, <A href="varff.html">VARFF</a></li> ');
 document.write(' <li> Files handling variables, <A href="varfil.html">VARFIL</a></li>  ');
 document.write(' <li> Geometry builder + symmetry related variables, <A href="vargeo.html">VARGEO</a></li> ');
@@ -15,6 +16,6 @@ document.write(' <li> Response Function variables, <A href="varrf.html">VARRF</a
 document.write(' <li> Structure optimization variables, <A href="varrlx.html">VARRLX</a></li> ');
 document.write(' <li> Wannier90 interface variables, <A href="varw90.html">VARW90</a></li> ');
 document.write(' </ul>');
-document.write(' See also the <a href="../users/spacegrouphelpfile.html">Space group table</a> ');
+document.write(' See also the <a href="../../users/spacegrouphelpfile.html">Space group table</a> ');
 document.write(' <hr> ');
 document.write(' ');
