@@ -199,7 +199,6 @@ def clear():
             if MD_file == None:
                 sys.exit(0)
               
-     
     
 #---------------------------------#       
 #  __  __    _    ___ _   _       #
@@ -283,6 +282,7 @@ while i<size_input :
             j=0
             size_calculation=len(input_data[input_name])
             choice = 4
+            showData(MD_file,units)
     else:
         print " "
         print "your options are:"
@@ -594,8 +594,7 @@ while i<size_input :
                     print  fname 
                     print 'File save sucessful!'            
             
-            
-            
+
             elif choiceQuantitie == 9:
                     
                 PTOE = Analysis.PeriodicTableElement()
@@ -607,9 +606,9 @@ while i<size_input :
                 else:
                     for i in range(2):
                         stringChoice = 'Choose the atom' + str(i+1)+' ( '
-                        for j in range(len(nameAtom)):
-                            stringChoice += str(nameAtom[j])
-                            if j != len(nameAtom)-1:
+                        for k in range(len(nameAtom)):
+                            stringChoice += str(nameAtom[k])
+                            if k != len(nameAtom)-1:
                                 stringChoice += ' or '
                             else:
                                 stringChoice+= ' ):'
@@ -723,19 +722,5 @@ while i<size_input :
 
     if(input_file == True):
         i+=1
+
 print "Thank you for using Abinit Post Process, see you soon!"
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
