@@ -11,10 +11,11 @@ import sys,os,re
 import threading
 
 #GUI 
-import gui.graph as Graph
-import gui.vdos  as VDOS
-import gui.rdf   as RDF
-import gui.msd   as MSD
+import gui.graph  as Graph
+import gui.vdos   as VDOS
+import gui.rdf    as RDF
+import gui.msd    as MSD
+import gui.netcdf as NETCDF
 import gui.atom_pos   as POS
 #Reading
 import reading.read as Read
@@ -22,6 +23,7 @@ import reading.read as Read
 #Utility
 import utility.analysis as Analysis
 import utility.write    as Write
+import utility.writeHIST as WriteHIST
 import utility.global_variable as Var
 
 #Fortran code
@@ -215,10 +217,13 @@ class Netcdf_MD(QtGui.QWidget):
         self.position.setStatusTip('Show positions of all the particules for all the step in 2D ')
         self.connect(self.position, QtCore.SIGNAL('clicked()'), self.showPosition)
 
-        self.MSD = QtGui.QPushButton('MSD', self)
-        self.MSD.setStatusTip('Show graphic of Means Squared Deplacement')
+        self.MSD = QtGui.QPushButton('MSD(beta)', self)
+        self.MSD.setStatusTip('Show graphic of Means Squared Displacement(beta)')
         self.connect(self.MSD, QtCore.SIGNAL('clicked()'), self.showMSD)        
 
+        self.netcdf = QtGui.QPushButton('save netcdf', self)
+        self.netcdf.setStatusTip('Save molecular dynamics in necdf format')
+        self.connect(self.netcdf, QtCore.SIGNAL('clicked()'), self.showNetcdf)
 
         self.xyz = QtGui.QPushButton('save .xyz', self)
         self.xyz.setStatusTip('Save position in .xyz format')
@@ -238,6 +243,7 @@ class Netcdf_MD(QtGui.QWidget):
         self.box4layout.addWidget(self.RDF,3,2)
         self.box4layout.addWidget(self.MSD,3,3)
         self.box4layout.addWidget(self.position,2,3)
+        self.box4layout.addWidget(self.netcdf,2,5)
         self.box4layout.addWidget(self.xyz,3,5)
         self.layout.addWidget(self.box2,1,0,5,1)
         self.layout.addWidget(self.box3,1,1,5,1)
@@ -428,7 +434,7 @@ class Netcdf_MD(QtGui.QWidget):
                     pass
                 else:
                     fname += '.xyz'
-                pos   = self.file1.getPos()   * 0.5291772085936 # Angstrom
+                pos   = self.file1.getXCart()   * 0.5291772085936 # Angstrom
                 acell = self.file1.getAcell() * 0.5291772085936 # Angstrom
                 typat = self.file1.getTypat()
                 znucl = self.file1.getZnucl()
@@ -436,7 +442,21 @@ class Netcdf_MD(QtGui.QWidget):
                 self.showMessage("file save in "+ str(fname))
         except:
             self.showError("Unable to save in xyz format ")
-            
+
+    def showNetcdf(self):
+        if self.file1.isGoodFile():
+            try:
+                fname = QtGui.QFileDialog.getSaveFileName(self,"Export data in xyz format",Var.path(), "HIST file (*_HIST)")
+                if (fname !=""):
+                    if 'HIST' in fname.split('_'):
+                        pass
+                    else:
+                        fname += '_HIST'
+                    WriteHIST.writeHIST(self.file1,fname,self.ni,self.nf)
+                    self.showMessage("file save in "+ str(fname))
+            except:
+                self.showError("Unable to save your simulation ")
+             
     def verifStep(self):
         if self.file1.isGoodFile():
             ni = self.sbni.value()
@@ -631,12 +651,12 @@ class Netcdf_MD(QtGui.QWidget):
             pass
 
         try:
-            if(units):
-                self.pos.units = self.units
-                self.pos.updateGraph()
-            else:
-                self.pos.file = self.file1
-                self.changeMode()
+#            if(units):
+#                self.pos.units = self.units
+#                self.pos.updateGraph()
+#            else:
+            self.pos.file = self.file1
+            self.changeMode()
         except:
             pass 
                             
