@@ -88,8 +88,8 @@ class MolecularDynamicFile:
 #--------METHODS--------------#
 #-----------------------------#
   def read_ascii(self):
-      #if 1==1:
-      try:
+      if 1==1:
+      #try:
 
           #Read the number of image:
           message = "Unable to read the number of image "
@@ -145,11 +145,18 @@ class MolecularDynamicFile:
                                           + self.namefile2 + '  | awk \'{print $1,$2,$3 }\'| sed \'/--/d\' | sed \'/Cartesian velocities (vel)/d\' ' )
           self.vel = np.reshape(np.array(temp.split()[0:self.n_image*self.Nbtime*self.natom*3],dtype=float),(self.Nbtime,self.n_image,self.natom,3))
 
-          #Reading position of particules:
-          message = "Unable to read the position of particules"
+          #Reading position of particules (Cartesian):
+          message = "Unable to read the position of particules (cart)"
           temp = commands.getoutput(' grep -A '+str(self.natom)+'  \"Cartesian coordinates\" '\
                                         + self.namefile2 + '  | awk \'{print $1,$2,$3 }\'| sed \'/--/d\' | sed \'/Cartesian coordinates/d\' ' )
-          self.pos = np.reshape(np.array(temp.split()[0:self.n_image*self.Nbtime*self.natom*3],dtype=float),(self.Nbtime,self.n_image,self.natom,3))
+          self.xcart = np.reshape(np.array(temp.split()[0:self.n_image*self.Nbtime*self.natom*3],dtype=float),(self.Nbtime,self.n_image,self.natom,3))
+
+          #Reading position of particules (Reduced):
+          message = "Unable to read the position of particules (red)"
+          temp = commands.getoutput(' grep -A '+str(self.natom)+'  \"Reduced coordinates\" '\
+                                        + self.namefile2 + '  | awk \'{print $1,$2,$3 }\'| sed \'/--/d\' | sed \'/Reduced coordinates/d\' ' )
+          
+          self.xred = np.reshape(np.array(temp.split()[0:self.n_image*self.Nbtime*self.natom*3],dtype=float),(self.Nbtime,self.n_image,self.natom,3))
 
           #Reading stress of particules:
           message = "Unable to read stress of particules"
@@ -183,8 +190,6 @@ class MolecularDynamicFile:
           temp = commands.getoutput('grep -P -A' + str(int(max(self.typat))/3) +' \'amu\s(.*)\' '+self.namefile2+' | sed \'s/amu//\'')
           self.amu = np.array(temp.split(), dtype=float)
 
-
-
           #Reading znucl :
           message = "Unable to read znucl"
           temp = commands.getoutput('grep  \"znucl \" '+self.namefile2+' | sed \'s/znucl//\'')
@@ -213,13 +218,13 @@ class MolecularDynamicFile:
 
         #-----------------------------------------------------#
 
-      except:    
-          self.goodFile = False
-          print "Error, " +message+ ", please use netcdf file "
-          try:
-              QtGui.QMessageBox.critical(None,"Warning","Error, " +message+ ", please use netcdf file ")#
-          except :
-              pass;
+#      except:    
+#          self.goodFile = False
+#          print "Error, " +message+ ", please use netcdf file "
+#          try:
+#              QtGui.QMessageBox.critical(None,"Warning","Error, " +message+ ", please use netcdf file ")#
+#          except :
+#              pass;
         
  
   def read_netcdf(self):
@@ -233,7 +238,7 @@ class MolecularDynamicFile:
 
       #if 1==1:
       try :
-         #Set the two netcdf_file variable with the HOST and ncfile:
+          #Set the two netcdf_file variable with the HOST and ncfile:
           self.hist_file = netcdf.netcdf_file(str(self.namefile1), 'r')
           self.nc_file   = netcdf.netcdf_file(str(self.namefile2), 'r')
 
@@ -266,9 +271,20 @@ class MolecularDynamicFile:
           
           #Reading potential energy:
           self.E_pot = getattr(self.hist_file.variables['etotal'],module)
+
+          #Reading potential energy:
+          self.E_kin_ion = getattr(self.hist_file.variables['ekin'],module)
+
+          #Reading potential energy:
+          self.mdtime = getattr(self.hist_file.variables['mdtime'],module)
             
           #Reading position of particules:
-          self.pos = np.reshape(getattr(self.hist_file.variables['xcart'],module),(self.Nbtime,self.n_image,self.natom,3))
+          self.xcart = np.reshape(getattr(self.hist_file.variables['xcart'],module),(self.Nbtime,self.n_image,self.natom,3))
+          self.xred = np.reshape(getattr(self.hist_file.variables['xred'] ,module),(self.Nbtime,self.n_image,self.natom,3))
+
+          #Reading forces of particules:
+          self.fcart = np.reshape(getattr(self.hist_file.variables['fcart'],module),(self.Nbtime,self.n_image,self.natom,3))
+          self.fred  = np.reshape(getattr(self.hist_file.variables['fred'] ,module),(self.Nbtime,self.n_image,self.natom,3))
           
           #Reading velocity of particules:
           self.vel = np.reshape(getattr(self.hist_file.variables['vel'],module),(self.Nbtime,self.n_image,self.natom,3))
@@ -282,11 +298,9 @@ class MolecularDynamicFile:
               self.dtion = 100
 
           #Reading znucl of particules:
-          self.znucl = getattr(self.nc_file.variables['znucl'],module)
-            
+          self.znucl = getattr(self.nc_file.variables['znucl'],module)            
           #----------------------------------------------------
           self.goodFile = True
-
           
           #-----------Calculation of some quantities------------#
           #-----------not available in the output File----------# 
@@ -370,11 +384,14 @@ class MolecularDynamicFile:
       # return Boolean True/False according to the HIST and NC file:
       return self.goodFile
 
-  def getNameFile(self):
+  def getNameFile(self,fullpath=False):
       # return string with the name of the file
       if self.goodFile:
-          name = self.namefile2.split('/')
-          return name[len(name)-1]
+          if fullpath :
+              return self.namefile2
+          else:
+              name = self.namefile2.split('/')
+              return name[len(name)-1]
       else:
           return 0
         
@@ -398,6 +415,13 @@ class MolecularDynamicFile:
           return self.typat
       else:
           return 0
+
+  def getNTypat(self):
+      # return the number of type of particules
+      if self.goodFile:
+          return np.max(self.typat)
+      else:
+          return 0
       
   def getAmu(self):
       # return 1D array with the atomic mass
@@ -466,14 +490,47 @@ class MolecularDynamicFile:
       else:
           return 0
 
-  def getPos(self,image = 1):
-      # return 3D array with the position (x, y, z) for all particule at each time ( pos[t][at][x,y,z]) for one image
+  def getXCart(self,image = 1):
+      # return 3D array with the position (x, y, z) for all particule at each time ( pos[t][at][x,y,z]) for one image in cartesian coordiantes
       # between ni and nf
       if self.goodFile:
           if self.n_image == 1:
-              return self.pos[self.ni:self.nf,image-1] # Temporarily because pos[t=0] = 0. put (ni-1) for start slicing at 0
+              return self.xcart[self.ni:self.nf,image-1] # Temporarily because pos[t=0] = 0. put (ni-1) for start slicing at 0
           elif self.n_image != 1 :
-              return self.getPosCentroid()
+              return self.getXCartCentroid()
+      else:
+          return 0
+
+  def getXRed(self,image = 1):
+      # return 3D array with the position (x, y, z) for all particule at each time ( pos[t][at][x,y,z]) for one image in reduced coordinates
+      # between ni and nf
+      if self.goodFile:
+          if self.n_image == 1:
+              return self.xred[self.ni:self.nf,image-1] # Temporarily because pos[t=0] = 0. put (ni-1) for start slicing at 0
+          elif self.n_image != 1 :
+              return self.getXCartCentroid(xred=True)
+      else:
+          return 0
+
+  def getFCart(self,image = 1):
+      # return 3D array with the forces (x, y, z) for all particule at each time ( pos[t][at][x,y,z]) for one image in cartesian coordinates
+      # between ni and nf
+      if self.goodFile:
+          if self.n_image == 1:
+              return self.fcart[self.ni:self.nf,image-1] # Temporarily because pos[t=0] = 0. put (ni-1) for start slicing at 0
+          elif self.n_image != 1 :
+              return self.getXCartCentroid(xred=True)
+      else:
+          return 0
+
+  def getFRed(self,image = 1):
+      # return 3D array with the forces (x, y, z) for all particule at each time ( f[t][at][x,y,z]) for one image in reduced coordinates
+      # between ni and nf
+      if self.goodFile:
+          if self.n_image == 1:
+              return self.fred[self.ni:self.nf,image-1] # Temporarily because pos[t=0] = 0. put (ni-1) for start slicing at 0
+          elif self.n_image != 1 :
+              return self.getXCartCentroid(xred=True)
       else:
           return 0
 
@@ -485,11 +542,14 @@ class MolecularDynamicFile:
       else:
           return 0
 
-  def getPosCentroid(self):
+  def getXCartCentroid(self,xred=False):
       # return 3D array with the position (x, y, z) for all particule at each time ( pos[t][at][x,y,z]) for the centroid
       # between ni and nf
       if self.goodFile:
-          return np.mean(self.pos[self.ni:self.nf,:],axis=1) # Temporarily because pos[t=0] = 0. put (ni-1) for start slicing at 0
+          if(xred):
+              return np.mean(self.xred[self.ni:self.nf,:],axis=1) # Temporarily because pos[t=0] = 0. put (ni-1) for start slicing at 0
+          else:
+              return np.mean(self.xcart[self.ni:self.nf,:],axis=1) # Temporarily because pos[t=0] = 0. put (ni-1) for start slicing at 0
       else:
           return 0
 
@@ -500,6 +560,19 @@ class MolecularDynamicFile:
           return self.E_pot[self.ni-1:self.nf-1]   #Temporarily. put (nf) for finish slicing at 299,
       else:                                        #Skip the last step to get the same array size (see vel and pos).
           return 0
+      
+  def getMdtime(self):
+      if self.goodFile :
+          return self.mdtime[self.ni-1:self.nf-1]   #Temporarily. put (nf) for finish slicing at 299,
+      else:                                        #Skip the last step to get the same array size (see vel and pos).
+          return 0
+
+  def getE_kin_ion(self):
+      # return 1D array with the Energy KINetic ionic (Ha)
+      if self.goodFile :
+          return self.E_kin_ion[self.ni-1:self.nf-1]   #Temporarily. put (nf) for finish slicing at 299,
+      else:                                        #Skip the last step to get the same array size (see vel and pos).
+          return 0
         
   def getE_kin(self):
       # return 1D array with the kinetic energy (Ha)
