@@ -210,46 +210,40 @@ class RDF:
 
         file = pfile
 
-        pos = file.getPos()            # position of atom
+        pos = file.getXCart()          # position of atom (cartesian)
 
-        inc = pstep                    #incrementation
+        inc = pstep                    # incrementation
 
-        acell = file.getAcell()        #acell
-        a = acell[0,0]
-        b = acell[0,1]
-        c = acell[0,2]
+        acell = file.getAcell()        # acell
+        a = acell[:,0]
+        b = acell[:,1]
+        c = acell[:,2]
+        a_max = max(acell[:,0])
+        b_max = max(acell[:,1])
+        c_max = max(acell[:,2])
 
         rprim = zeros((3,3))           # primitives vectors of the cell
-        rprim[0] = file.getRPrim()[0,0] / a
-        rprim[1] = file.getRPrim()[0,1] / b
-        rprim[2] = file.getRPrim()[0,2] / c
+        rprim[0] = file.getRPrim()[0,0]
+        rprim[1] = file.getRPrim()[0,1]
+        rprim[2] = file.getRPrim()[0,2]
 
-        alpha = (arccos((rprim[0][0]*rprim[1][0]+rprim[0][1]*rprim[1][1]+rprim[0][2]*rprim[1][2])\
-        /sqrt((rprim[0][0]**2+rprim[0][1]**2+rprim[0][2]**2)*(rprim[1][0]**2+rprim[1][1]**2+rprim[1][2]**2))))
-        beta  = (arccos((rprim[0][0]*rprim[2][0]+rprim[0][1]*rprim[2][1]+rprim[0][2]*rprim[2][2])\
-        /sqrt((rprim[0][0]**2+rprim[0][1]**2+rprim[0][2]**2)*(rprim[2][0]**2+rprim[2][1]**2+rprim[2][2]**2))))
-        gamma = (arccos((rprim[2][0]*rprim[1][0]+rprim[2][1]*rprim[1][1]+rprim[2][2]*rprim[1][2])\
-        /sqrt((rprim[2][0]**2+rprim[2][1]**2+rprim[2][2]**2)*(rprim[1][0]**2+rprim[1][1]**2+rprim[1][2]**2))))
-
-
-        #a *= sin(alpha)
-        #b *= sin(beta)
-        #c *= sin(gamma)
         rho = 1/file.getVol()        # density
 
         if box >= 0:
 
-            rmax = (1/2.+box)*min(a,b,c)           # maximum radius
+            f = (1/2.+box)           # factor of maximum radius
             
         else: # box < 0
             
-            rmax = 3**0.5/2.*min(a,b,c)
+            f = 3**0.5/2.
 
-        deltaR = pdr                           # delta r  (Bohr)
+        rmax = f*min(a_max,b_max,c_max)
 
-        maxbin = int((rmax/deltaR))            # number of iteration
+        deltaR = pdr                      # delta r  (Bohr)
 
-        typat = file.getTypat()              # Type of particule
+        maxbin = int((rmax/deltaR))       # number of iteration
+
+        typat = file.getTypat()           # Type of particule
 
         if atom1 == 0:
             indexAtom1 = array([i for i,x in enumerate(typat) if x == 1 and 2],dtype=int) + 1
@@ -268,13 +262,13 @@ class RDF:
 
         if mode == 0:
 
-            self.g = topo.pair_distribution(nbtime,pos,rmax,inc,a,b,c,deltaR,rho,indexAtom1,indexAtom2,self.r)
+            self.g = topo.pair_distribution(nbtime,pos,rprim,f,inc,a,b,c,deltaR,rho,indexAtom1,indexAtom2,self.r)
 
             self.r += deltaR/2.
 
         else: #mode == 1
 
-            nba = len(pos[0])                     # number of atom
+            nba = len(pos[0])                   # number of atom
 
             it = (int(nbtime/inc)+1)*nba*(nba-1)
 
@@ -309,24 +303,26 @@ class DEC:
 
         file = pfile
 
-        pos = file.getPos()            # position of atom
+        pos = file.getXCart()            # position of atom (cartesian)
 
         nei = n
 
         inc = step                    #incrementation
 
         acell = file.getAcell()        #acell
-        a = acell[0,0]
-        b = acell[0,1]
-        c = acell[0,2]
+        a_max = max(acell[:,0])
+        b_max = max(acell[:,1])
+        c_max = max(acell[:,2])
 
         if box >= 0:
 
-            rmax = (1/2.+ box)*min(a,b,c)           # maximum radius
-            
+            f = (1/2.+ box)           # factor of maximum radius
+
         else: # box < 0
             
-            rmax = 3**0.5/2.*min(a,b,c)
+            f = 3**0.5/2.
+
+        rmax = f*min(a_max,b_max,c_max)
 
         deltaR = pdr                           # delta r  (Bohr)
 
@@ -372,14 +368,19 @@ class ADF:
 
         file = pfile
 
-        pos = file.getPos()            # position of atom
+        pos = file.getXCart()            # position of atom (cartesian)
 
         inc = pstep                    #incrementation
 
         acell = file.getAcell()        #acell
-        a = acell[0,0]
-        b = acell[0,1]
-        c = acell[0,2]
+        a = acell[:,0]
+        b = acell[:,1]
+        c = acell[:,2]
+
+        rprim = zeros((3,3))           # primitives vectors of the cell
+        rprim[0] = file.getRPrim()[0,0]
+        rprim[1] = file.getRPrim()[0,1]
+        rprim[2] = file.getRPrim()[0,2]
 
         deltaTheta = pdt                           # delta Theta  (degres)
 
@@ -402,7 +403,7 @@ class ADF:
 
         self.t = arange(maxbin)*deltaTheta
 
-        self.a = topo.angular_distribution(nei,nbtime,pos,inc,a,b,c,deltaTheta,indexAtom1,indexAtom2,self.t)
+        self.a = topo.angular_distribution(nei,nbtime,pos,rprim,inc,a,b,c,deltaTheta,indexAtom1,indexAtom2,self.t)
 
     def getTheta(self):
         return self.t
@@ -421,7 +422,7 @@ class NDF:
 
         # This class aim is to calculate the Neighbor Distrubution Function
         # the parameters are :
-        #    - nei gives the neith neibours we consider. The deconvolution of the RDF is made with nei = 0
+        #    - nei gives the neith neibours we consider.
         #    - pfile  is the output file (ascii/netcdf)
         #    - atom1 give the number of the atom in typat
         #    - pdt   : give the dtheta for the calculation
@@ -429,14 +430,19 @@ class NDF:
 
         file = pfile
 
-        pos = file.getPos()            # position of atom
+        pos = file.getXCart()            # position of atom (cartesian)
 
         inc = pstep                    #incrementation
 
         acell = file.getAcell()        #acell
-        a = acell[0,0]
-        b = acell[0,1]
-        c = acell[0,2]
+        a = acell[:,0]
+        b = acell[:,1]
+        c = acell[:,2]
+
+        rprim = zeros((3,3))           # primitives vectors of the cell
+        rprim[0] = file.getRPrim()[0,0]
+        rprim[1] = file.getRPrim()[0,1]
+        rprim[2] = file.getRPrim()[0,2]
 
         deltaR = pdr
 
@@ -459,7 +465,7 @@ class NDF:
 
         it = (int(nbtime/inc)+1)*nba*(nba-1)
 
-        self.m = topo.m_distribution(nei,nbtime,pos,inc,it,a,b,c,indexAtom1,indexAtom2)
+        self.m = topo.m_distribution(nei,nbtime,pos,rprim,inc,it,a,b,c,indexAtom1,indexAtom2)
 
         maxi = self.m[0]
 
@@ -500,12 +506,17 @@ class Proba:
 
         file = pfile
 
-        pos = file.getPos()            # position of atom
+        pos = file.getXCart()            # position of atom (cartesian)
 
         acell = file.getAcell()        #acell
-        a = acell[0,0]
-        b = acell[0,1]
-        c = acell[0,2]
+        a = acell[:,0]
+        b = acell[:,1]
+        c = acell[:,2]
+
+        rprim = zeros((3,3))           # primitives vectors of the cell
+        rprim[0] = file.getRPrim()[0,0]
+        rprim[1] = file.getRPrim()[0,1]
+        rprim[2] = file.getRPrim()[0,2]
 
         typat = file.getTypat()              # Type of particule
 
@@ -518,7 +529,7 @@ class Proba:
 
         self.n = arange(1,nei+1)
 
-        self.p = topo.probability(nei,nbtime,pos,a,b,c,indexAtom1,indexAtom1)
+        self.p = topo.probability(nei,nbtime,pos,rprim,a,b,c,indexAtom1,indexAtom1)
 
     def getN(self):
         return self.n
@@ -543,7 +554,7 @@ class MSD:
 
         file = pfile
 
-        pos = file.getPos()            # position of atom
+        pos = file.getXCart()          # position of atom (cartesian)
 
         typat = file.getTypat()        # Type of particule
         
