@@ -1,7 +1,7 @@
 /* abi_xc_vdw.h */
 
 /*
- * Copyright (C) 2012-2014 ABINIT Group (Yann Pouillon)
+ * Copyright (C) 2012-2016 ABINIT Group (Yann Pouillon)
  *
  * This file is part of the ABINIT software package. For license information,
  * please see the COPYING file in the top-level directory of the ABINIT source
@@ -38,18 +38,18 @@
        call vdw_df_write_func(ABI_FUNC,mode) NEWLINE \
        call sentinel(2,mode,__FILE__,funcname,__LINE__)
 #  else
-#    define VDWXC_DBG_ENTER(mode) \
+#    define VDWXC_DBG_ENTER(mode,funcname) \
        call vdw_df_write_func(ABI_FUNC,mode) NEWLINE \
        call sentinel(1,mode)
-#    define VDWXC_DBG_EXIT(mode) \
+#    define VDWXC_DBG_EXIT(mode,funcname) \
        call vdw_df_write_func(ABI_FUNC,mode) NEWLINE \
        call sentinel(2,mode)
 #  endif
 
 #else
 
-#  define VDWXC_DBG_ENTER(mode)
-#  define VDWXC_DBG_EXIT(mode)
+#  define VDWXC_DBG_ENTER(mode,funcname)
+#  define VDWXC_DBG_EXIT(mode,funcname)
 
 #endif
 
