@@ -1,7 +1,7 @@
 /* cuda_header.h */
 
 /*
- * Copyright (C) 2008-2014 ABINIT Group (MMancini)
+ * Copyright (C) 2008-2016 ABINIT Group (MMancini)
  *
  * This file is part of the ABINIT software package. For license information,
  * please see the COPYING file in the top-level directory of the ABINIT source
@@ -16,13 +16,17 @@
 
 //#include <cutil.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <unistd.h>
 #include <cufft.h>
 #include <cuda_runtime.h>
 #include <cuda.h>
-#include "config.h"
 #include "cuda_common.h"
+#include <stdio.h>
 
+#if defined HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #ifdef HAVE_GPU_CUDA_DP
   typedef double cureal;
@@ -54,7 +58,7 @@ static inline char* cufftGetErrorString(cufftResult res)
   char* message=(char*)malloc(25*sizeof(char));
 
   switch(res){
-  case CUFFT_SUCCESS:
+  case CUFFT_SUCCESS :
     sprintf(message,"CUFFT_SUCCESS");
     break;
   case CUFFT_INVALID_PLAN :
