@@ -14,13 +14,16 @@ is_multi_parallel = True
 
 #: List of input files
 inp_files = [
-"tA.in",
-"tP.in",
-"tQ.in",
-"tR.in",
-"tT.in",
-"tX.in",
-"tY.in",
-"-tAG.in", # Disabled
-"tAI.in",
+"t01.in",
+"t21.in",
+"t22.in",
+"t24.in",
+"t25.in",
+"t26.in",
+"t42.in",
+"t49.in",
+"t51.in",
+"t62.in",
+"t69.in", 
+"t99.in",
 ]
