@@ -34,6 +34,7 @@ inp_files = [
 "t27.in",
 "t28.in",
 "t29.in",
+"t30.in",
 "t41.in",
 "-t48.in", # MPI-IO, not tested here!
 "-t49.in", # MPI-IO, not tested here!
@@ -48,12 +49,14 @@ inp_files = [
 "t59.in",
 "t60.in",
 "-t61.in", # disabled for now
+"-t62.in", # disabled for now
 "t71.in",
 "t72.in",
 "t73.in",
 "t74.in",
 "t75.in",
 "t76.in",
+"t77.in",
 "t91.in",
 "t92.in",
 "t93.in",
