@@ -1,8 +1,7 @@
 #!/usr/bin/env python
-from __future__ import division, print_function
+from __future__ import print_function, division, absolute_import #, unicode_literals
 
 import sys
-import os
 import abc
 import difflib
 import tempfile
@@ -797,8 +796,8 @@ class FileDiffer(object):
             if not options.verbose:
                 # Print the most important difference
                 print(diff_list[0])
-            else
-                print(str(diff_list)
+            else:
+                print(str(diff_list))
 
             html.add_lines_and_diff(i, header, s1, s2, diff_list, verbose=verbose)
 
