@@ -1,37 +1,30 @@
-#!/usr/bin/env python
+from __future__ import print_function, division, absolute_import #, unicode_literals
 
 import os
 import sys
 import time
 
-from subprocess import Popen, PIPE
-from socket import gethostname
-from os.path import join as pj, abspath as absp, exists as pexists, basename, isfile
-
-from subprocesswithtimeout import SubProcessWithTimeout
+from subprocess import Popen
+from .subprocesswithtimeout import SubProcessWithTimeout
 
 try:
     from ConfigParser import SafeConfigParser, NoOptionError
-except ImportError: # The ConfigParser module has been renamed to configparser in Python 3
+except ImportError:
+    # The ConfigParser module has been renamed to configparser in Python 3
     from configparser import SafeConfigParser, NoOptionError
 
+import logging
+logger = logging.getLogger(__name__)
+
 __version__ = "0.1"
-__author__  = "Matteo Giantomassi"
+__author__ = "Matteo Giantomassi"
 
 __all__ = [
-"JobRunner",
-"TimeBomb",
-"OMPEnvironment",
+    "JobRunner",
+    "TimeBomb",
+    "OMPEnvironment",
 ]
 
-_debug = False
-#_debug = True
-if _debug:
-    def dbg_print(*args): print(args)
-else:
-    def dbg_print(*args): pass
-
-#############################################################################################################
 
 CFG_KEYWORDS = {
 # keyword             : (parser, default value i.e. NO MPI, section, description)
@@ -47,17 +40,24 @@ CFG_KEYWORDS = {
 #"info"              : (str, "", "mpi", "String with optional information"),
 }
 
+
+def is_string(s):
+    try:
+        s + "hello"
+        return True
+    except TypeError:
+        return False
+
+
 def mpicfg_parser(fname, defaults=None):
-    """Parse the configuration file that describes the test."""
-    dbg_print("Parsing [MPI] section in file : " + str(fname))
+    """Parse the configuration file with MPI options."""
+    logger.debug("Parsing [MPI] section in file : " + str(fname))
 
     parser = SafeConfigParser(defaults)
     parser.read(fname)
 
-    ############################################
     # Read variables needed to handle the job.
-    ############################################
-    d = dict()
+    d = {}
 
     for key, tup in CFG_KEYWORDS.items():
         line_parser = tup[0]
@@ -66,100 +66,114 @@ def mpicfg_parser(fname, defaults=None):
             try:
                 d[key] = parser.get(section, key)
             except NoOptionError:
-                d[key] = tup[1] # Section exists but option is not specified. Use default value.
+                # Section exists but option is not specified. Use default value.
+                d[key] = tup[1]
         else:
-            d[key] = tup[1] # Section does not exist. Use default value.
+            # Section does not exist. Use default value.
+            d[key] = tup[1]
 
         # Process the line
         try:
-            d[key] = line_parser( d[key] )
+            d[key] = line_parser(d[key])
         except:
             raise ValueError("Wrong line: key = " + str(key) + " d[key] = " + str(d[key]) )
 
     return d
 
-#############################################################################################################
+
 class JobRunnerError(Exception):
+    """Exceptions raised by `Jobrunner`."""
 
     def __init__(self, return_code, cmd, run_etime, prev_errmsg=None):
+        """
+        Args:
+            return_code: Return code of the subprocess
+            cmd: Command executed
+            run_etime: Elapsed-time
+            prev_errmsg: Previous error message.
+        """
+        # This is needed for pickle
+        super(JobRunnerError, self).__init__()
         self.return_code = return_code
         self.cmd = cmd
         self.run_etime = run_etime
         self.prev_errmsg = prev_errmsg
 
     def __str__(self):
-        #return "\n".join( [str(k) + " : " + str(v) for (k,v) in self.__dict__.items()] )
         string = "Command %s\n returned exit_code: %s\n" % (self.cmd, self.return_code)
         if self.prev_errmsg:
             string += "Previous exception: %s" % self.prev_errmsg
+
         return string
 
-#############################################################################################################
 
 class JobRunner(object):
     """Base Class used to manage the execution of jobs in an MPI environment."""
+    Error = JobRunnerError
 
     @classmethod
     def fromdict(cls, kwargs, ompenv=None, timebomb=None):
-        d = dict()
-        d["ompenv"] = ompenv
-        d["timebomb"] = timebomb
+        """Initialize the object from a dictionary"""
+        d = dict(ompenv=ompenv, timebomb=timebomb)
         d.update(kwargs)
+
         return cls(d)
 
     @classmethod
     def fromfile(cls, fname, timebomb=None):
+        """Initialize the object from a INI configuration file."""
         d = mpicfg_parser(fname)
         d["ompenv"] = OMPEnvironment.from_file(fname, allow_empty=True)
         d["timebomb"] = timebomb
+
         return cls(d)
 
     @classmethod
     def sequential(cls, ompenv=None, timebomb=None):
-        d = dict()
-        d["ompenv"] = ompenv
-        d["timebomb"] = timebomb
-        return cls(d)
+        """Build a simple `JobRunner` for sequential runs."""
+        return cls(dict(ompenv=ompenv, timebomb=timebomb))
 
     @classmethod
     def generic_mpi(cls, ompenv=None, use_mpiexec=False, timebomb=None):
+        """Build a simple `JobRunner` for MPI runs (assumes some default values)."""
         # It should work, provided that the shell environment is properly defined.
-        d = {}
+        d = dict(ompenv=ompenv, timebomb=timebomb)
+
         if use_mpiexec:
             d["mpirun_np"] = "mpiexec -np"
         else:
             d["mpirun_np"] = "mpirun -np"
         #d["mpirun_extra_args"] = ""
-        d["ompenv"] = ompenv
-        d["timebomb"] = timebomb
+
         return cls(d)
 
     def __init__(self, dic):
-        self.__debug = _debug
 
         self.exceptions = []
 
-        for (k, v) in dic.items():
+        for k, v in dic.items():
             if k not in self.__dict__:
                 self.__dict__[k] = v
             else:
-                err_msg = " key %s is already in self.__dict__, cannot overwrite" % k
-                raise ValueError(err_msg)
+                raise ValueError("key %s is already in self.__dict__, cannot overwrite" % k)
 
-        if (self.has_poe and self.has_mpirun):
+        if self.has_poe and self.has_mpirun:
             raise ValueError("poe and mpirun are mutually exclusive")
 
     def __str__(self):
-        if self.__debug:
-            return "\n".join([str(k) + " : " + str(v) for (k, v) in self.__dict__.items()] )
+        #return "\n".join([str(k) + " : " + str(v) for (k, v) in self.__dict__.items()] )
 
         string = ""
         for key in CFG_KEYWORDS:
             attr = getattr(self, str(key), None)
             if attr: string += "%s = %s\n" % (key, attr)
-        if string: string = "[MPI setup]\n" + string
 
-        if self.has_ompenv: string += "[OpenMP]\n" + str(self.ompenv)
+        if string:
+            string = "[MPI setup]\n" + string
+
+        if self.has_ompenv:
+            string += "[OpenMP]\n" + str(self.ompenv)
+
         return string
 
     def set_timebomb(self, timebomb):
@@ -169,10 +183,12 @@ class JobRunner(object):
             self.timebomb = timebomb
 
     def set_valgrind_cmdline(self, cmdline):
+        """Set the command line options to be passed to VALGRIND."""
         self.valgrind_cmdline = cmdline
 
     @property
     def has_valgrind(self):
+        """True if we are running the code with VALGRIND."""
         return hasattr(self, "valgrind_cmdline")
 
     def build_valgrind_parser(self):
@@ -181,40 +197,71 @@ class JobRunner(object):
         return MemcheckParser()
 
     def set_debugger(self, debugger):
+        """Set the debugger."""
         self.debugger = debugger
 
     @property
     def has_debugger(self):
+        """True if we are running the executable under the control of a debugger."""
         return hasattr(self, "debugger")
 
+    def set_perf_command(self, perf_command):
+        """Set the perf command to be used."""
+        self.perf_command = perf_command
+
+    @property
+    def has_perf(self):
+        """True if we are profiling the run with perf."""
+        return hasattr(self, "perf_command")
+
     @property
     def has_mpirun(self):
-        return ( hasattr(self, "mpirun_np") and
-                 bool(getattr(self, "mpirun_np")) )
+        """True if we are running a MPI job with mpirun"""
+        return hasattr(self, "mpirun_np") and bool(getattr(self, "mpirun_np"))
 
     @property
     def has_poe(self):
-        return ( hasattr(self, "poe") and
-                 bool(getattr(self, "poe")) )
+        """True if are using IBM poe for MPI executions."""
+        return hasattr(self, "poe") and bool(getattr(self, "poe"))
 
     @property
     def has_timebomb(self):
-        return ( hasattr(self, "timebomb") and
-                 bool(getattr(self, "timebomb")) )
+        """
+        True if we are running the job under the control of
+        an application that will enforce a timeout.
+        """
+        return hasattr(self, "timebomb") and bool(getattr(self, "timebomb"))
 
     @property
     def has_ompenv(self):
-        return ( hasattr(self, "ompenv") and
-                 bool(getattr(self, "ompenv")) )
+        """True if we are using OpenMP."""
+        return hasattr(self, "ompenv") and bool(getattr(self, "ompenv"))
 
     def set_ompenv(self, ompenv):
+        """Set the value of the OpenMP env variables."""
         if self.has_ompenv:
             raise ValueError("ompenv is already defined")
         else:
             self.ompenv = ompenv
 
     def run(self, mpi_nprocs, bin_path, stdin_fname, stdout_fname, stderr_fname, cwd=None):
+        """
+        Args:
+            mpi_nprocs:
+                Number of MPI nodes.
+            bin_path:
+                Path of the executable.
+            stdin_fname:
+                Input file
+            stdout_fname:
+                Output file
+            stderr_fname:
+                Error file
+            cwd:
+                cd to cwd before launching the job.
 
+        Set self.retcode
+        """
         env = os.environ.copy()
         if self.has_ompenv: env.update(self.ompenv)
 
@@ -223,24 +270,29 @@ class JobRunner(object):
         if self.has_valgrind: 
             valcmd = "valgrind --tool=%s " % self.valgrind_cmdline
 
+        # Perf command
+        perf_cmd = ""
+        if self.has_perf:
+            perf_cmd = "perf %s " % self.perf_command
+
         if self.has_mpirun:
-            args = [self.mpirun_np, str(mpi_nprocs), valcmd, bin_path, 
+            args = [perf_cmd, self.mpirun_np, str(mpi_nprocs), valcmd, bin_path, 
                    "<", stdin_fname, ">", stdout_fname, "2>", stderr_fname]
 
         elif self.has_poe:
             # example ${poe} abinit ${poe_args} -procs 4
-            # no support for valgrind or debugger here since poe uses a weird syntax for command line options.
+            # no support for valgrind, debugger or perf here since poe uses a weird syntax for command line options.
             args = [self.poe, bin_path, self.poe_args, " -procs "+str(mpi_nprocs),
                     " <", stdin_fname, ">", stdout_fname, "2>", stderr_fname]
         else:
             assert mpi_nprocs == 1
-            args = [valcmd, bin_path, "<", stdin_fname, ">", stdout_fname, "2>",stderr_fname]
+            args = [perf_cmd, valcmd, bin_path, "<", stdin_fname, ">", stdout_fname, "2>",stderr_fname]
 
         if self.has_debugger:
             # Use completely different syntax if we are running under the control of gdb.
             # mpirun -np 2 xterm -e gdb fftprof --command=dbg_file
 
-            # Get the working directory (warning: I assume that stderr_fname is an absolute path.
+            # Get the working directory (warning: I assume that stderr_fname is an absolute path).
             workdir = os.path.dirname(stderr_fname)
 
             dbg_filepath = os.path.join(workdir, "dbg_commands")
@@ -257,28 +309,29 @@ class JobRunner(object):
         cmd = " ".join(args)
 
         #if self.has_valgrind: print("Invoking valgrind:\n %s" % cmd)
-        if self.__debug: print "About to execute command:\n" + cmd
+        logger.debug("About to execute command:\n" + cmd)
 
         start_time = time.time()
-        ret_code = None
+        self.retcode = -1
 
         try:
             if self.has_timebomb:
-                (p, ret_code) = self.timebomb.run(cmd, shell=True, cwd=cwd, env=env)
+                p, self.retcode = self.timebomb.run(cmd, shell=True, cwd=cwd, env=env)
             else:
                 p = Popen(cmd, shell=True, cwd=cwd, env=env)
-                ret_code = p.wait()
+                self.retcode = p.wait()
 
             run_etime = time.time() - start_time
 
-            if ret_code != 0:
-                exc = JobRunnerError(ret_code, " ".join(args), run_etime)
+            if self.retcode != 0:
+                exc = JobRunnerError(self.retcode, " ".join(args), run_etime)
+                logger.debug(str(exc))
                 self.exceptions.append(exc)
 
         except:
             run_etime = time.time() - start_time
             prev_errmsg = str(sys.exc_info()[1])
-            exc = JobRunnerError(ret_code, " ".join(args), run_etime, prev_errmsg)
+            exc = JobRunnerError(self.retcode, " ".join(args), run_etime, prev_errmsg)
             self.exceptions.append(exc)
 
         return run_etime
@@ -348,10 +401,11 @@ class MemcheckParser(BaseValgrindParser):
         ]
 
         # Inspect the next len(keys) line (memleak section)
-        errors  = {}
+        errors = {}
         for key, line in zip(keys, fh): 
             bytes = fragile_parser(key, line)
-            if bytes: errors[key] = bytes
+            if bytes:
+                errors[key] = bytes
 
         # Get total number of errors.
         key = "ERROR SUMMARY:" 
@@ -365,6 +419,7 @@ class MemcheckParser(BaseValgrindParser):
         self._error_report = ""
         if errors: self._error_report = str(errors)
 
+
 class TimeBomb(object):
 
     def __init__(self, timeout, delay=.05, exec_path=None):
@@ -375,7 +430,7 @@ class TimeBomb(object):
     def run(self, args,
             bufsize=0, executable=None, stdin=None, stdout=None, stderr=None, preexec_fn=None,
             close_fds=False, shell=False, cwd=None, env=None, universal_newlines=False, startupinfo=None, creationflags=0):
-        "Same interface as Popen"
+        """Same interface as Popen."""
 
         try:
 
@@ -384,8 +439,8 @@ class TimeBomb(object):
                 # timeout exec is available.
                 #
                 if self.timeout > 0.:
-                    dbg_print("Using timeout function: " + self.exec_path)
-                    if isinstance(args, str):
+                    logger.debug("Using timeout function: " + self.exec_path)
+                    if is_string(args):
                         args = " ".join([self.exec_path, str(self.timeout), args])
                     else:
                         args = [self.exec_path, str(self.timeout)] + args
@@ -402,15 +457,15 @@ class TimeBomb(object):
                 # timeout exec is NOT available.
                 #
                 if self.timeout > 0.0:
-                    dbg_print("Using SubprocesswithTimeout and timeout_time : "+str(self.timeout))
+                    logger.debug("Using SubprocesswithTimeout and timeout_time : "+str(self.timeout))
                     p = SubProcessWithTimeout(self.timeout, delay=self.delay)
 
-                    (p, ret_code) = p.run(args,
+                    p, ret_code = p.run(args,
                         bufsize=bufsize, executable=executable, stdin=stdin, stdout=stdout, stderr=stderr, preexec_fn=preexec_fn,
                         close_fds=close_fds, shell=shell, cwd=cwd, env=env, universal_newlines=universal_newlines, startupinfo=startupinfo,
                         creationflags=creationflags)
                 else:
-                    dbg_print("Using Popen (no timeout_time)")
+                    logger.debug("Using Popen (no timeout_time)")
                     p = Popen(args,
                               bufsize=bufsize, executable=executable, stdin=stdin, stdout=stdout, stderr=stderr, preexec_fn=preexec_fn,
                               close_fds=close_fds, shell=shell, cwd=cwd, env=env, universal_newlines=universal_newlines, startupinfo=startupinfo,
@@ -418,14 +473,17 @@ class TimeBomb(object):
 
                     ret_code = p.wait()
 
-            return (p, ret_code)
+            return p, ret_code
 
         except:
             raise
 
-#############################################################################################################
+
 class OMPEnvironment(dict):
-    #: OMP3 variables. see https://computing.llnl.gov/tutorials/openMP/#EnvironmentVariables"
+    """
+    OpenMP variables. 
+    see https://computing.llnl.gov/tutorials/openMP/#EnvironmentVariables
+    """
     _keys = [
        "OMP_SCHEDULE",
        "OMP_NUM_THREADS",
@@ -461,6 +519,7 @@ class OMPEnvironment(dict):
 
     @classmethod
     def from_file(cls, fname, allow_empty=False):
+        """Initialize the object from file."""
         parser = SafeConfigParser()
         parser.read(fname)
 
@@ -477,7 +536,8 @@ class OMPEnvironment(dict):
         for key in parser.options("openmp"):
             if key.upper() not in OMPEnvironment._keys:
                 err_msg += "unknown option %s, maybe a typo" % key
-        if err_msg: raise ValueError(err_msg)
+        if err_msg:
+            raise ValueError(err_msg)
 
         for key in OMPEnvironment._keys:
             try:
@@ -492,6 +552,3 @@ class OMPEnvironment(dict):
             raise ValueError("Refusing to return with an empty dict")
 
         return inst
-
-#############################################################################################################
-#if __name__ == "__main__":
