@@ -3,6 +3,7 @@ A multi-producer, multi-consumer queue.
 Taken from http://hg.python.org/cpython/file/2.7/Lib/Queue.py
 Modified by MG to be py2.4 compliant: notify_all => notifyALL
 """
+from __future__ import print_function, division, absolute_import #, unicode_literals
 
 from time import time as _time
 try:
@@ -305,4 +306,4 @@ if __name__ == "__main__":
 
     q.join()
     #q.join_with_timeout(0.000001)
-    print 'All inputs found their way to 1.  Queue is empty and all processing complete.'
+    print('All inputs found their way to 1.  Queue is empty and all processing complete.')
