@@ -1,17 +1,16 @@
-#!/usr/bin/env python
-from __future__ import division
+from __future__ import print_function, division, absolute_import #, unicode_literals
 
 import os
 import sys
 import shutil 
 import tempfile
+import warnings
 
-from os.path import join as pj, abspath as absp
 from subprocess import Popen, PIPE, call
-from warnings import warn
+
 
 __version__ = "0.1"
-__author__  = "Matteo Giantomassi"
+__author__ = "Matteo Giantomassi"
 
 __all__ = [
     "RestrictedShell",
@@ -21,9 +20,10 @@ __all__ = [
 
 # Helper functions
 
+
 def patch(fromfile, tofile):
     """
-    Use diff and patch to patch tofile.  
+    Use the unix tools diff and patch to patch tofile.  
     Returns 0 if success
     """
     # Temporary patch file.
@@ -38,7 +38,7 @@ def patch(fromfile, tofile):
     retcode = os.system(diff_cmd)
 
     #if retcode >= 2: 
-    #    warn("%s returned %s, won't patch!" % (diff_cmd, retcode) )
+    #    warnings.warn("%s returned %s, won't patch!" % (diff_cmd, retcode) )
     #    return retcode
 
     # Keep a backup copy of tofile.
@@ -47,12 +47,13 @@ def patch(fromfile, tofile):
 
     #print(open(tmp, "r").readlines())
     #patch_cmd = "patch -p1 -i %s -o %s" % (tmp, tofile)
-    patch_cmd = "patch -p0 < %s" % tmp
+    #patch_cmd = "patch -p0 < %s" % tmp
+    patch_cmd = "cp %s %s" % (fromfile, tofile)
     #print(patch_cmd)
 
     retcode = os.system(patch_cmd)
     if retcode != 0:
-        warn("%s returned %s, reverting to original file  %s" % (patch_cmd, retcode))
+        warnings.warn("%s returned %s, reverting to original file" % (patch_cmd, retcode))
         shutil.move(bkp, tofile)
         return retcode
 
@@ -68,7 +69,8 @@ def unzip(gz_fname, dest=None):
     """decompress a gz file."""
     import gzip
 
-    if not gz_fname.endswith(".gz"): raise ValueError("%s should end with .gz" % gz_fname)
+    if not gz_fname.endswith(".gz"):
+        raise ValueError("%s should end with .gz" % gz_fname)
 
     try:
         gz_fh = gzip.open(gz_fname, 'rb')
@@ -85,19 +87,23 @@ def unzip(gz_fname, dest=None):
 
 
 def touch(fname):
+    """Emulate unix touch."""
     try:
-        open(fname,"w").close()
+        open(fname, "w").close()
         return 0
     except:
         raise IOError("trying to create file = %s" % fname)
 
 
 def tail_file(fname, n, aslist=False):
-    """Assumes a unix-like system."""
-
+    """Emulate unix tail. Assumes a unix-like system."""
     args = ["tail", "-n " + str(n), fname]
 
-    p = Popen(args, shell=False, stdout=PIPE, stderr=PIPE)
+    if sys.version_info >= (3, 0):
+        p = Popen(args, shell=False, stdout=PIPE, stderr=PIPE, universal_newlines=True)
+    else:
+        p = Popen(args, shell=False, stdout=PIPE, stderr=PIPE)
+
     ret_code = p.wait()
 
     if ret_code != 0:
@@ -111,9 +117,9 @@ def tail_file(fname, n, aslist=False):
 
 def which(program):
     """
-    python version of the unix tool which
-    locate a program file in the user's path
-    Return None if program cannot be found.
+    python version of the unix tool which locate a program file in the user's path
+    Return:
+        None if program cannot be found.
     """
     def is_exe(fpath):
         return os.path.isfile(fpath) and os.access(fpath, os.X_OK)
@@ -131,14 +137,6 @@ def which(program):
     return None
 
 
-from string import maketrans, punctuation
-_table = maketrans("","")
-
-def strip_punct(s):
-    "Remove punctuation characters from string s."
-    return s.translate(_table, punctuation)
-
-
 def tonumber(s):
     """Convert string to number, raise ValueError if s cannot be converted."""
     # Duck test. Much more readable than the ugly strfltrem routine in fldiff.pl
@@ -153,16 +151,16 @@ def tonumber(s):
 
 
 def nums_and_text(line):
-    "split line into (numbers, text)."
+    """split line into (numbers, text)."""
     tokens = line.split()
-    text    = ""
-    numbers = list()
+    text = ""
+    numbers = []
     for tok in tokens:
         try:
             numbers.append( tonumber(tok) )
         except ValueError:
             text += " " + tok
-    return (numbers, text)
+    return numbers, text
 
 
 class RShellError(Exception):
@@ -170,12 +168,20 @@ class RShellError(Exception):
 
 
 class RestrictedShell(object):
+    """
+    This object executes a restricted set of shell commands.
+    It's main goal is to provide a restricted access to the
+    computing environment as we want to avoid executing 
+    arbitrary code passed through the TEST_INFO sections.
+
+    At present, it supports rm, cp, mv and touch 
+    """
     _key2command = {
-    # key      (function,   nargs)
-    "rm"    : (shutil.rmtree, 2),
-    "cp"    : (shutil.copy,   2),
-    "mv"    : (shutil.move,   2),
-    "touch":  (touch,       1),
+        # key (function,   nargs)
+        #"rm": (shutil.rmtree, 2),
+        "cp": (shutil.copy,   2),
+        "mv": (shutil.move,   2),
+        "touch":  (touch,     1),
     }
 
     Error = RShellError
@@ -186,9 +192,9 @@ class RestrictedShell(object):
         self.exceptions = []
 
         self.prefix2dir = {
-          "i" : absp(inp_dir),
-          "w" : absp(workdir),
-          "p" : absp(psps_dir),
+            "i": os.path.abspath(inp_dir),
+            "w": os.path.abspath(workdir),
+            "p": os.path.abspath(psps_dir),
         }
 
     def empty_exceptions(self):
@@ -199,7 +205,7 @@ class RestrictedShell(object):
         Don't raise exceptions since python threads get stuck.
         Exceptions are stored in self.exceptions
         """
-        #print "executing %s" % string
+        #print("executing %s" % string)
         _key2command = RestrictedShell._key2command
 
         tokens = string.split()
@@ -212,34 +218,63 @@ class RestrictedShell(object):
             nargs = len(args)
             if nargs != expected_nargs:
                 err_msg = " Too many arguments, cmd = %s, args = %s " % (cmd, args)
-                self.exceptions.append(RShellError(err_msg))
+                self.exceptions.append(self.Error(err_msg))
                 return
         except:
             err_msg = "Not able to interpret the string: %s " % string
-            self.exceptions.append(RShellError(err_msg))
+            self.exceptions.append(self.Error(err_msg))
             return
 
         new_args = []
-        for (pref, arg) in zip(pres, args):
-            new_args.append( pj(self.prefix2dir[pref], arg)  )
+        for pref, arg in zip(pres, args):
+            new_args.append(os.path.join(self.prefix2dir[pref], arg))
         #print "new_args: ",new_args
 
         try:
             if nargs == 1:
+                # Touch
                 assert pres == "w"
                 return cmd(new_args[0])
+
             elif nargs == 2:
-                return cmd(new_args[0], new_args[1])
+                # Copy or Move
+                src, dest = new_args[0], new_args[1]
+
+                # If src does not exist, look for the netcdf version.
+                # and change the extension of dest accordingly.
+                # This trick is needed so that we can run the same 
+                # test both with Fortran-IO as well as with Netcdf
+                # without having to use two different TEST_INFO sections..
+                if key in ("cp", "mv"):
+                    if not os.path.exists(src):
+                        print("File %s does not exist, will try netcdf version" % src)
+                        if os.path.exists(src + "-etsf.nc"):
+                            src += "-etsf.nc"
+                            dest += "-etsf.nc"
+                        elif os.path.exists(src + ".nc"):
+                            src += ".nc"
+                            dest += ".nc"
+                        else:
+                            exc = self.Error("Cannot find neither %s nor netcdf versions in workdir" % src)
+                            return self.exceptions.append(exc)
+                        print("Fortran file not found, will do:", key, src, dest)
+
+                # Execute command
+                return cmd(src, dest)
+
             else:
                 raise NotImplementedError("nargs = %s is too large" % nargs)
+
         except:
             import sys
             err_msg = "Executing: " + string + "\n" + str(sys.exc_info()[1])
-            self.exceptions.append(RShellError(err_msg))
+            self.exceptions.append(self.Error(err_msg))
 
 
-# Python cookbook, #475186
 def stream_has_colours(stream):
+    """
+    True if stream supports colours. Python cookbook, #475186
+    """
     if not hasattr(stream, "isatty"):
         return False
 
@@ -250,18 +285,20 @@ def stream_has_colours(stream):
         curses.setupterm()
         return curses.tigetnum("colors") > 2
     except:
-        return False # guess false in case of error
+        # guess false in case of error
+        return False
 
 
 class StringColorizer(object):
-    colours={"default":"",
-             "blue"  :  "\x1b[01;34m",
-             "cyan" :  "\x1b[01;36m",
-             "green": "\x1b[01;32m",
-             "red"  : "\x1b[01;31m",
-             # lighting colours.
-             #"lred":    "\x1b[01;05;37;41m"
-             }
+    colours = {
+        "default": "",
+        "blue":  "\x1b[01;34m",
+        "cyan": "\x1b[01;36m",
+        "green": "\x1b[01;32m",
+        "red": "\x1b[01;31m",
+        # lighting colours.
+        #"lred":    "\x1b[01;05;37;41m"
+        }
 
     def __init__(self, stream):
         self.has_colours = stream_has_colours(stream)
@@ -277,17 +314,28 @@ class StringColorizer(object):
             return string
 
 
+def prompt(question):
+    if sys.version_info >= (3, 0):
+        my_input = input
+    else:
+        # python 2.x.
+        my_input = raw_input
+    
+    return my_input(question)
+
+
 def user_wants_to_exit():
+    """Interactive problem, return False if user entered `n` or `no`."""
     try:
-        answer = raw_input("Do you want to continue [Y/n]")
+        answer = prompt("Do you want to continue [Y/n]")
     except EOFError:
-        #print "exc"
         return True
 
     return answer.lower().strip() in ["n", "no"]
 
 
 class Editor(object):
+    """Python interface to text editors."""
     def __init__(self, editor=None):
         if editor is None:
             self.editor = os.getenv("EDITOR", "vi")
@@ -303,57 +351,56 @@ class Editor(object):
             retcode = call([self.editor, fname, "+%s" % str(lineno)])
 
         if retcode != 0:
-            import warnings
             warnings.warn("Error while trying to edit file: %s" % fname)
+
         return retcode
 
     def edit_files(self, fnames, ask_for_exit=True):
+        """
+        Edit a list of files, if assk_for_exit is True, we ask
+        whether the user wants to exit from the cycle at each iteration.
+        """
         exit_status = 0
         for idx, fname in enumerate(fnames):
             exit_status = self.edit_file(fname)
             if ask_for_exit and idx != len(fnames)-1 and user_wants_to_exit():
                 break
+
         return exit_status
 
 
-def input_from_editor(message=None):
-    if message: print message,
-    from tempfile import mkstemp
-    fd, fname = mkstemp(text=True)
-
-    Editor().edit_file(fname)
-
-    filobj = open(fname)
-    string = filobj.read()
-    filobj.close()
-    return string
-
-
 class PatcherError(Exception):
     """Base class for errors raised by patcher"""
 
 
 class Patcher(object):
+    """Python interface to differt/patcher tools."""
     Error = PatcherError
 
+    # Interactive programs (I known) with support for patches.
     interactive_patchers = [
-      "vimdiff",
-      "kdiff3",
-      "tkdiff",
+        "vimdiff",
+        "kdiff3",
+        "tkdiff",
     ]
 
-    auto_patchers = [
-      "patch",
-    ]
+    # The revered unix tool for automatic patches.
+    auto_patchers = ["patch"]
 
     known_patchers = interactive_patchers + auto_patchers
 
     def __init__(self, patcher=None):
+        """
+        Args:
+            patcher:
+                string with the name of the utility we want to use 
+                for generating/applying patchers
+                If patcher is None, we use the applications specified 
+                in the env variables PATHCHER or vimdiff if $PATHCHER is 
+                not defined
+        """
         if patcher is None:
             self.patcher = os.getenv("PATCHER", "vimdiff")
-            #if which(self.patcher) is None:
-            #    print "kdiff3 not found, setting patcher to vimdiff"
-            #    self.patcher = "vimdiff"
 
         else:
             if patcher not in Patcher.known_patchers: 
@@ -365,23 +412,25 @@ class Patcher(object):
 
     @property
     def is_interactive(self):
+        """True if this is an interactive patcher."""
         return self.patcher in Patcher.interactive_patchers
 
     def patch(self, fromfile, tofile):
-
+        """Patch a file."""
         if self.patcher == "patch":
             try: 
                 return patch(fromfile, tofile)
-            except Exception, exc:
+            except Exception as exc:
                 raise self.Error("%s: trying to patch  %s %s:\n%s" % (self.patcher, fromfile, tofile, str(exc)))
 
         else:
             try:
                 return call([self.patcher, fromfile, tofile])
-            except Exception, exc:
+            except Exception as exc:
                 raise self.Error("%s: trying to patch  %s %s:\n%s" % (self.patcher, fromfile, tofile, str(exc)))
 
     def patch_files(self, fromfiles, tofiles):
+        """Patch a list of files."""
         assert len(fromfiles) == len(tofiles)
         exit_status = 0
 
@@ -391,7 +440,7 @@ class Patcher(object):
             return exit_status
 
         if not self.is_interactive:
-            ans = raw_input("Will patch %d files. Continue [y/n]" % nfiles)
+            ans = prompt("Will patch %d files. Continue [y/n]" % nfiles)
             if ans.lower() not in ["y", "yes"]:
                 print("Exit requested by user. Nothing is changed.")
                 return 0
@@ -399,7 +448,7 @@ class Patcher(object):
         for idx, (fro, to) in enumerate(zip(fromfiles, tofiles)):
 
             if self.is_interactive:
-                #print "from %s, to %s" % (fro, to)
+                #print("from %s, to %s" % (fro, to))
                 exit_status = self.patch(fro, to)
                 if idx != (nfiles-1) and user_wants_to_exit(): 
                     break
@@ -423,9 +472,13 @@ def pprint_table(table, out=sys.stdout, rstrip=False):
     Prints out a table of data, padded for alignment
     Each row must have the same number of columns.
 
-    out: Output stream (file-like object)
-    table: The table to print. A list of lists.
-    rstrip: if true, trailing withespaces are removed from the entries.
+    Args:
+        out: 
+            Output stream (file-like object)
+        table: 
+            The table to print. A list of lists.
+        rstrip: 
+            if true, trailing withespaces are removed from the entries.
     """
     def max_width_col(table, col_idx):
         """Get the maximum width of the given column index"""
@@ -481,8 +534,9 @@ r"""
                                                         V
 """
 
+
 def ascii_scream():
-    return r"""\
+    return r"""
 ---;;;;;;;-----'''''''''``'  --- `'  .,,ccc$$hcccccc,.  `' ,;;!!!'``,;;!!'
 ;;;;,,.,;-------''''''' ,;;!!-    .zJ$$$$$$$$$$$$$$$$$$$c,. `' ,;;!!!!' ,;
   ```'    -;;;!'''''-  `.,..   .zJ$$$$$$$$$$$$$$$$$$$$$$$$$$c, `!!'' ,;!!'
@@ -528,5 +582,53 @@ cc                   `$$$c`?        ?$.`$$hc, cd$$F ,$'  $$$$$$     ;!!
 c,        ;,        `?$$$$P           !!>             .
 """
 
-#if __name__ == "__main__":
-    #print(ascii_scream())
+
+def ascii_abinit():
+    return r"""
+                          :':                                                                       
+                         ''''                                                                       
+                         '''':                                                                      
+                         '''';                                                                      
+                        ''''''                                                                      
+                        ''''''                                                    ,`                
+                       '`,''''                                                  ::                  
+                      :'  ''''                                                ::.                   
+                      '   ''''                                             :::,                     
+                     ''   '''',                                         ::::                        
+                     '    '''''                                      ::::,                          
+    `,              '`    :''''                                  .:::::                             
+  ;;,              ;'      ''''                   :'`        `:::::.                                
+ ';;;              '       ''''`                 ;'''`   ,::::::                                    
+ ;;;;;:           ''       '''''      '          ''''':::::.                                        
+  ;;;;;;;,       .'         ''''     ,'      `:;:'''';,                  ''                         
+    ';;;;;;;;':`                   ``:';;;;;;;;:, '';                    ''                         
+       ,';;;;;;;;;;;;;;;;;;;;;;;;;;;;;':,`                               ''                         
+              `.,:::;;;:::,,.`       ,'  .;;`      :   ,   ,,       :    ''                         
+               .+            `''''   ,'`''''''     '   '.''''''     '  ''''''                       
+               ''             ;''''  ,'''    ''    '   '''`   ''    '    ''                         
+              ''               ''''' ,''      ''   '   ''      ''   '    ''                         
+              ',                 ,:  ,'       `'   '   ''      :'   '    ''                         
+             ''                      ,'        '   '   '.       '   '    ''                         
+            :'                        '        '   '   '.       '   '    ''                         
+            ''                        '.      ;'   '   '.       '   '    ''                         
+           ''                         ''      ';   '   '.       '   '    ''                         
+           '.                          '''  '''    '   '.       '   '    ,',                        
+          ''                            ;'''':     '   '.       '   '     ''';                      
+         ''                                                                                         
+         ':                                                        :  .       `````````             
+        ''                                                      `,'   `:::::::::::::::::::::::,`    
+        ;                                               `:;;;;;;;:'                     .,::::::::  
+                                                  .;;;;;;:.       ,   '                      `:::::.
+                                             .;;;;;:`              .+;                          :::,
+                                         :;;;;:`                                                ,:` 
+                                     :;;;;.                                                   ..    
+                                  ;;;;,                                                             
+                               ;;;;`                                                                
+                            .;;;                                                                    
+                          :;;                                                                       
+                         ;.                                                                         
+                       ;
+"""
+
+if __name__ == "__main__":
+    print(ascii_abinit())
