@@ -1,15 +1,18 @@
 #!/usr/bin/env python
+from __future__ import print_function, division, absolute_import #, unicode_literals
 
 import sys
 import os
 import platform
-import cPickle as cpkl
 import shelve
 
-from os.path import join as pj, abspath as absp, exists as pexists, isfile, basename
+from os.path import join as pj, abspath as absp, basename
 from socket import gethostname
 from warnings import warn
 
+import logging
+logger = logging.getLogger(__name__)
+
 try:
     from ConfigParser import SafeConfigParser, NoOptionError
 except ImportError: # The ConfigParser module has been renamed to configparser in Python 3
@@ -25,17 +28,20 @@ sys.path.insert(0,pack_dir)
 import tests
 
 __version__ = "0.3"
-__author__  = "Matteo Giantomassi"
+__author__ = "Matteo Giantomassi"
 
 _my_name = basename(__file__) + "-" + __version__
 
 abitests = tests.abitests
 abenv = tests.abenv
 
+from pymods import termcolor
 from pymods.testsuite import BuildEnvironment
 from pymods.jobrunner import TimeBomb, JobRunner, OMPEnvironment
 
 # Helper functions
+
+
 def lazy__str__(func):
     "Lazy decorator for __str__ methods"
     def oncall(*args, **kwargs):
@@ -43,6 +49,7 @@ def lazy__str__(func):
         return "\n".join( [ str(k) + " : " + str(v) for (k,v) in self.__dict__.items()] )
     return oncall
 
+
 def _yesno2bool(string):
     string = string.lower().strip().replace('"',"").replace("'","")
     if string == "yes":
@@ -52,11 +59,10 @@ def _yesno2bool(string):
     else:
         raise ValueError("Cannot interpret string: %s" % string)
 
+
 def _str2list(string): 
     return [s.strip() for s in string.split(",") if s]
 
-#def nop(token):
-#    return token
 
 class TestBot(object):
     """
@@ -74,8 +80,8 @@ class TestBot(object):
       "type"             : ("",   str),  # "ref" if this slave is the reference slave, e.g. testf
       "ncpus"            : (None, int),  # Max number of CPUs that can be used by TestBot
       "mpi_prefix"       : ("",   str),  # MPI runner.
-      "mpirun_np"        : ("",   str),
-      "omp_num_threads"  : (0,    int), # Number of OpenMp threads to be used
+      "mpirun_np"        : ("",   str),  # String used to execute `mpirun -n#NUM`
+      "omp_num_threads"  : (0,    int),  # Number of OpenMp threads to be used
       "enable_mpi"       : (None, _yesno2bool),
       "enable_openmp"    : (None, _yesno2bool),
       "poe"              : ("", str),
@@ -86,6 +92,7 @@ class TestBot(object):
       "cygwin_dir"       : (None, str),
       "runmode"          : ("static", str),
       "keywords"         : ("", _str2list),   # String with the keywords that should be selected/ignored
+      "etsf_check"       : ("no", _yesno2bool),  # yes to activate validation of netcdf files produced by Abinit.
     }
 
     def __init__(self, testbot_cfg=None):
@@ -110,9 +117,10 @@ class TestBot(object):
           "with_tdirs",
           "without_tdirs",
           "timeout_time",
-	      "cygwin_dir",
+          "cygwin_dir",
           "runmode",
           "keywords",
+          "etsf_check",
         ]
 
         for attr in attrs2read:
@@ -125,9 +133,9 @@ class TestBot(object):
             if value is None:
                 # Write out the cfg file and raise
                 for section in parser.sections():
-                    print "[" + section + "]"
+                    print("[" + section + "]")
                     for opt in parser.options(section):
-                        print opt + " = " + parser.get(section, opt)
+                        print(opt + " = " + parser.get(section, opt))
                 err_msg = "Mandatory option %s is not declared" % attr
                 raise ValueError(err_msg)
 
@@ -138,8 +146,18 @@ class TestBot(object):
             raise ValueError(err_msg)
 
         (system, node, release, version, machine, processor) = platform.uname()
-        print "Running on %s -- slave %s -- system %s -- ncpus %s -- Python %s -- %s" % (
-          gethostname(), self.slavename, system, self.ncpus, platform.python_version(), _my_name)
+        print("Running on %s -- slave %s -- system %s -- ncpus %s -- Python %s -- %s" % (
+               gethostname(), self.slavename, system, self.ncpus, platform.python_version(), _my_name))
+
+        # Set the logger level.
+        # loglevel is bound to the string value obtained from the command line argument. 
+        # Convert to upper case to allow the user to specify --loglevel=DEBUG or --loglevel=debug
+        #numeric_level = getattr(logging, options.loglevel.upper(), None)
+        numeric_level = getattr(logging, "ERROR", None)
+
+        if not isinstance(numeric_level, int):
+            raise ValueError('Invalid log level: %s' % options.loglevel)
+        logging.basicConfig(level=numeric_level)
 
         # Read testfarm configuration file.
         build_examples = abenv.apath_of(pj("config", "specs", "testfarm.conf"))
@@ -167,7 +185,8 @@ class TestBot(object):
         #  d[attr] = parse(d[attr])
 
         # 2 )Initialize the jobrunner.
-        self.build_env = build_env = BuildEnvironment(os.curdir,cygwin_instdir=self.__dict__["cygwin_dir"])
+        self.build_env = build_env = BuildEnvironment(os.curdir, cygwin_instdir=self.__dict__["cygwin_dir"])
+        self.build_env.set_buildbot_builder(self.slavename)
 
         if build_env.has_bin("timeout") and self.timeout_time > 0:
             # We can run executables under the control of timeout.c
@@ -177,23 +196,23 @@ class TestBot(object):
             warn("Cannot find timeout executable at: %s" % build_env.path_of_bin("timeout"))
             timebomb = TimeBomb(self.timeout_time)
 
-        print "Initalizing JobRunner for sequential runs."
+        print("Initalizing JobRunner for sequential runs.")
         self.seq_runner = JobRunner.sequential(timebomb=timebomb)
-        print self.seq_runner
+        print(self.seq_runner)
 
         if self.has_mpi:
-            print "Initalizing MPI JobRunner from self.__dict__"
+            print("Initalizing MPI JobRunner from self.__dict__")
             self.mpi_runner = JobRunner.fromdict(self.__dict__, timebomb=timebomb)
-            print self.mpi_runner
+            print(self.mpi_runner)
 
         if self.omp_num_threads > 0:
-            print "Initalizing OMP environment with omp_num_threads %d " % self.omp_num_threads
+            print("Initalizing OMP environment with omp_num_threads %d " % self.omp_num_threads)
             omp_env = OMPEnvironment(OMP_NUM_THREADS = self.omp_num_threads)
             self.seq_runner.set_ompenv(omp_env)
             if self.has_mpi: self.mpi_runner.set_ompenv(omp_env)
 
         self.targz_fnames = []
-        print self
+        print(self)
 
         # Initalize the table to store the final results.
         # The table include all the abinit tests (also those that will be skipped)
@@ -202,7 +221,7 @@ class TestBot(object):
         database = abitests.get_database()
         res_table = database.init_result_table()
         self.summary = TestBotSummary(res_table)
-        #print self.summary
+        #print(self.summary)
 
     @lazy__str__
     def __str__(self): pass
@@ -210,7 +229,7 @@ class TestBot(object):
     @property
     def has_mpi(self):
         """True if we have the MPI runner"""
-        return (bool(self.mpirun_np) or bool(self.poe))
+        return bool(self.mpirun_np) or bool(self.poe)
 
     @property
     def has_openmp(self):
@@ -220,6 +239,7 @@ class TestBot(object):
     def run_tests_with_np(self, mpi_nprocs, suite_args=None, runmode="static"):
         """
         Run the tests specified by suite_args, using mpi_nprocs MPI processors
+        Returns: (nfailed, npassed, nexecuted)
         """
         omp_nthreads = max(self.omp_num_threads, 1)
         py_nthreads = self.ncpus // (mpi_nprocs * omp_nthreads)
@@ -239,33 +259,37 @@ class TestBot(object):
         # Run the tests.
         if self.has_openmp:
             msg = "Running ntests = %s, MPI_nprocs = %s, OMP_nthreads %s, py_nthreads = %s..." % (
-                     test_suite.full_length, mpi_nprocs, omp_nthreads, py_nthreads)
+                  test_suite.full_length, mpi_nprocs, omp_nthreads, py_nthreads)
         else:
             msg = "Running ntests = %s, MPI_nprocs = %s, py_nthreads = %s..." % (
-                    test_suite.full_length, mpi_nprocs, py_nthreads)
-        print msg
+                  test_suite.full_length, mpi_nprocs, py_nthreads)
+        print(msg)
 
         runner = self.seq_runner
         if mpi_nprocs > 1: runner = self.mpi_runner
 
-        results = test_suite.run_tests(self.build_env, workdir, runner, 
-            nprocs=mpi_nprocs, 
-            nthreads=py_nthreads,
-            runmode=self.runmode
-        )
+        results = test_suite.run_tests(
+            self.build_env, workdir, runner,
+            nprocs=mpi_nprocs, nthreads=py_nthreads, runmode=self.runmode)
+            # Cannot use this option on the test farm because hdf5 is not thread-safe.
+            # See https://www.hdfgroup.org/hdf5-quest.html#tsafe
+            #etsf_check=self.etsf_check) 
+
+        if results is None:
+            print("Test suite is empty, returning 0 0 0 ")
+            return 0, 0, 0
 
         # Store the results in the summary table,
         # taking into account that an input file might be executed multiple times
         # with a different environment (MPI, OMP ...)
         run_info = {}
         #run_info = [self.build_env, workdir, runner, mpi_nprocs, py_nthreads]
-
         self.summary.merge_results(test_suite, run_info)
 
         # Push the location of the tarball file
         self.targz_fnames.append(results.targz_fname)
 
-        return (results.nfailed, results.npassed)
+        return results.nfailed, results.npassed, results.nexecuted
 
     def run(self):
         """
@@ -277,7 +301,8 @@ class TestBot(object):
         # else create a list of strings with the suites to (execute|exclude).
         suite_args = None
 
-        #XG130410 Crude hack, to avoid paral and mpiio test directories in case of enable_mpi=no in config/specs/testfarm.conf
+        # XG130410 Crude hack, to avoid paral and mpiio test directories
+        # in case of enable_mpi=no in config/specs/testfarm.conf
         if not self.has_mpi:
             suite_args = "paral- mpiio-".split()
 
@@ -289,19 +314,20 @@ class TestBot(object):
             suite_args = suite_args.split()
 
         # Run tests with 1 processor
-        #print suite_args
+        #print(suite_args)
         #runmode = "static"
+        #nexecuted = 0
 
         if self.runmode == "static":
             # Old mode: run all available tests with 1 MPI node here, 
             # then use MPI mode in mp_suites (paral/mpiio)
             np_list = [2, 4, 10, 24]
-            (nfailed, npassed) = self.run_tests_with_np(1, suite_args=suite_args, runmode=self.runmode)
+            nfailed, npassed, nexecuted = self.run_tests_with_np(1, suite_args=suite_args, runmode=self.runmode)
         else:
-            # New mode: run all available tests with 2 MPI nodes here, 
+            # New mode: run all available tests with 2 MPI procs here, 
             # then enter the mp_suites with [4, 10, 24] CPUs
             np_list = [4, 10, 24]
-            (nfailed, npassed) = self.run_tests_with_np(2, suite_args=suite_args, runmode=self.runmode)
+            nfailed, npassed, nexecuted = self.run_tests_with_np(2, suite_args=suite_args, runmode=self.runmode)
 
         if self.has_mpi:
             # Run the parallel tests in the multi-parallel suites.
@@ -319,16 +345,17 @@ class TestBot(object):
 
             for np in np_list:
                 if np > self.ncpus or not suite_args: continue
-                print "Running multi-parallel tests with %s MPI processors, suite_args %s" % (np, suite_args)
-                (para_nfailed, para_npassed) = self.run_tests_with_np(np, runmode="static", suite_args=suite_args)
-                #
+                print("Running multi-parallel tests with %s MPI processors, suite_args %s" % (np, suite_args))
+                para_nfailed, para_npassed, para_nexec = self.run_tests_with_np(np, runmode="static", suite_args=suite_args)
+
                 # Accumulate the counters.
                 nfailed += para_nfailed
                 npassed += para_npassed
+                nexecuted += para_nexec
 
         # TODO Collect tarball files.
-        for fname in self.targz_fnames:
-            print "got targz file: ",fname
+        #for fname in self.targz_fnames:
+        #    print("got targz file: ", fname)
 
         # Dump the table with the final results.
         # The buildbot master will upload it and the info will be used
@@ -339,16 +366,26 @@ class TestBot(object):
         table = self.summary.totable()
         pprint_table(table)
 
-        #self.summary.cpkl_dump("testbot_summary.cpkl")
+        self.summary.json_dump("testbot_summary.json")
         self.summary.shelve_dump("testbot_summary.shelve")
 
+        # Empty list of tests (usually due to the use of with, without options)
+        # Create file to signal this condition and return 0 
+        if nexecuted == 0:
+            print("No file found")
+            with open("__emptylist__", "w") as fh:
+                fh.write("nfailed = %d, npassed = %d, nexecuted %d" % (nfailed, npassed, nexecuted))
+                return 0
+
         # Status error depends on the builder type.
         if self.type == "ref":
             return nfailed + npassed  # Reference slave (all the tests must pass).
         else:
             return nfailed
 
+
 class TestBotSummary(object):
+    """Stores the final results of the tests performed by TestBot."""
 
     _possible_status = ["failed", "passed", "succeeded", "skipped"]
 
@@ -368,8 +405,8 @@ class TestBotSummary(object):
         """
         Return a table (list of lists whose elements are strings).
         The table has the format:
-        - [ suite_name1, suite_name2, ... ]
-        - [ stats1,      stats2,      ... ]
+            - [ suite_name1, suite_name2, ... ]
+            - [ stats1,      stats2,      ... ]
 
         where stats is given by "nfail/npass/nsucces"
         """
@@ -382,14 +419,13 @@ class TestBotSummary(object):
         row = []
         for suite_name in self:
             status, stats = self.status_of_suite(suite_name)
-            row.append( stats2string(stats) )
+            row.append(stats2string(stats))
         table.append(row)
 
         return table
 
     def merge_results(self, test_suite, run_info):
-
-        assert test_suite._executed
+        #assert test_suite._executed
         self.run_info = run_info
 
         for test in test_suite:
@@ -418,11 +454,11 @@ class TestBotSummary(object):
             self.passed.append(test.full_id)
 
     def __iter__(self):
-        "Iterate over the suite names in alphabetical order"
+        """Iterate over the suite names in alphabetical order."""
         for suite_name in self.suite_names(): yield suite_name
 
     def suite_names(self):
-        "List of suite names in alphabetical order"
+        """List of suite names in alphabetical order."""
         return sorted(self.res_table.keys())
 
     def status_of_suite(self, suite_name):
@@ -436,7 +472,7 @@ class TestBotSummary(object):
         # Initialize stats setting the keys to 0
         stats = dict.fromkeys(self._possible_status, 0)
 
-        for (test_id, d) in self.res_table[suite_name].items():
+        for test_id, d in self.res_table[suite_name].items():
             if "status" not in d: d["status"] = "skipped"
             stats[d["status"]] += 1
 
@@ -447,7 +483,7 @@ class TestBotSummary(object):
         else:
             raise RuntimeError("Wrong list of status values!")
 
-        return (suite_status, stats)
+        return suite_status, stats
 
     def shelve_dump(self, fname, protocol=0):
         """
@@ -455,26 +491,66 @@ class TestBotSummary(object):
         that will be transferred from the slave to the buildbot master.
         Use protocol=0 to avoid portability problems.
         """
+        def as_ascii(s):
+            #return str(s.encode("ascii", "ignore"))
+            return str(s)
+
         d = shelve.open(fname, protocol=protocol, flag="c")
 
         # list of strings with the name of the tests that (failed|passed)
-        d["failed"] = self.failed
-        d["passed"] = self.passed
-
-        d["summary_table"] = self.totable()
+        d[as_ascii("failed")] = self.failed
+        d[as_ascii("passed")] = self.passed
+        d[as_ascii("summary_table")] = self.totable()
 
         for suite_name in self:
-            print "---",suite_name, self.res_table[suite_name]
+            suite_name = as_ascii(suite_name)
+            print("---", suite_name, self.res_table[suite_name])
             if suite_name in d:
                 #raise KeyError("Cannot overwrite key %s" % suite_name)
-                print "Warning: About to overwrite key %s" % suite_name
+                print("Warning: About to overwrite key %s" % suite_name)
             d[suite_name] = self.res_table[suite_name]
 
         d.close()
 
-if __name__ == "__main__":
+    def json_dump(self, fname):
+        """
+        Save self.res_table, self.failed, self.passed in json format.
+        The file will be transferred from the slave to the buildbot master.
+        """
+        def as_ascii(s):
+            #return str(s.encode("ascii", "ignore"))
+            return str(s)
+
+        d = {}
+        # list of strings with the name of the tests that (failed|passed)
+        d[as_ascii("failed")] = self.failed
+        d[as_ascii("passed")] = self.passed
+        d[as_ascii("summary_table")] = self.totable()
+
+        for suite_name in self:
+            suite_name = as_ascii(suite_name)
+            print("---", suite_name, self.res_table[suite_name])
+            if suite_name in d:
+                #raise KeyError("Cannot overwrite key %s" % suite_name)
+                print("Warning: About to overwrite key %s" % suite_name)
+            d[suite_name] = self.res_table[suite_name]
+
+        import json
+        with open(fname, "wt") as fh:
+            json.dump(d, fh)
+
+
+def main():
     # Configuration file (hardcoded or from command line)
     testbot_cfg = None
-    if len(sys.argv) > 1: testbot_cfg = sys.argv[1]
+    if len(sys.argv) > 1:
+        testbot_cfg = sys.argv[1]
 
-    sys.exit(TestBot(testbot_cfg).run())
+    # Disable colors
+    termcolor.enable(False)
+
+    return TestBot(testbot_cfg).run()
+
+
+if __name__ == "__main__":
+    sys.exit(main())
