@@ -13,12 +13,12 @@ keywords = [
 is_multi_parallel = True
 
 subsuites = [
-#"tgswl", 
 "dfpt",
 "gspw",
-#"mbt",
+#"gswvl",
+"mbt",
 #"moldyn",
-#"string",
+"string",
 ]
 
 #: List of input files
@@ -31,24 +31,26 @@ inp_files = [
 "tdfpt_04.in",
 #"tdfpt_03PAW.in,"
 #"tdfpt_04PAW.in",
+#"tdmft_01.in",
+#"tdmft_02.in",
 "tgspw_01.in",
-#"tgspw_02.in",
-#"tgspw_03.in",
-#"tgspw_04.in",
+"tgspw_02.in",    # OK
+"tgspw_03.in",    # OK
+#"tgspw_04.in",   # Unstable because nstep=5 and bandpp: 2d iteration oscillates and fldiff does not handle it!
 #"tgspw_05.in",
-#"tmbt_1.in",
-#"tmbt_2.in",
-#"tmbt_3.in",
-#"tmbt_4.in",
-#"tmbt_5.in",
-#"tmoldyn_01.in",
+"tmbt_1.in",   # OK     
+"tmbt_2.in",   # OK
+"tmbt_3.in",   # OK
+"tmbt_4.in",   # OK
+#"tmbt_5.in",  
+#"tmoldyn_01.in",    # This is not stable. Does it use random number generators?
 #"tmoldyn_02.in",
 #"tmoldyn_03.in",
 #"tmoldyn_04.in",
 #"tmoldyn_05.in",
 #"tmoldyn_06.in",
 #"tmoldyn_07.in",
-#"tstring_01.in",
+"tstring_01.in",   # MPI Error in MPI_ALLREDUCE
 #"tstring_02.in",
 #"tstring_03.in",
 #"tstring_04.in",
