@@ -10,6 +10,7 @@ keywords = [
 
 subsuites = [
 "elast",
+"emp",
 "eph",
 "ffield",
 "nlo",
@@ -26,6 +27,10 @@ inp_files = [
 "telast_4.in",
 "telast_5.in",
 "telast_6.in",
+"temp_1.in",
+"temp_2.in",
+"temp_3.in",
+"temp_4.in",
 "teph_1.in",
 "teph_2.in",
 "teph_3.in",
@@ -38,7 +43,7 @@ inp_files = [
 "tffield_4.in",
 "tffield_5.in",
 "tffield_6.in",
-"-tffield_7.in",  # Disabled
+#"-tffield_7.in",  # Disabled
 "tnlo_1.in",
 "tnlo_2.in",
 "tnlo_3.in",
