@@ -1,5 +1,3 @@
-"""Global variables associated to the test suite."""
-
 #: List of CPP variables that should be defined in config.h in order to enable this suite.
 need_cpp_vars = [
 ]
@@ -23,6 +21,7 @@ subsuites = [
 "spin",
 "tddft",
 "udet",
+"fold2bloch",
 ]
 
 #: List of input files
@@ -90,4 +89,6 @@ inp_files = [
 "tudet_1.in",
 "tudet_2.in",
 "tudet_3.in",
+"tfold2bloch_1.in",
+"tfold2bloch_2.in",
 ]
