@@ -65,7 +65,6 @@ inp_files = [
 "t54.in",
 "t55.in",
 "t56.in",
-"-t57.in",  # ACDF is not supported anymore (requires mkmem ==0)
 "t58.in",
 "t59.in",
 "t60.in",
@@ -86,9 +85,6 @@ inp_files = [
 "t75.in",
 "t76.in",
 "t77.in",
-"t78.in",
-"t79.in",
-"t80.in",
 "-t81.in",  # moved to seq
 "-t82.in",  # moved to seq
 "t83.in",
